<?php declare(strict_types=1);

namespace App\Repository;

use App\Controller\DatabaseConnection;
use App\Entity\Domain;
use PDO;
use PDOException;

/**
 *
 */
class DomainRepository
{
	public function __construct(private DatabaseConnection $databaseConnection)
	{}
	
	
	/**
	 * @return array
	 */
	public function findAll(): array
	{
		$domains = [];
		$sql = "
            SELECT id, name, panel_id, a, aaaa
            FROM " . DatabaseConnection::TABLE_DOMAINS . "
            ORDER BY name";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->execute();
			while ($result = $statement->fetch(mode: PDO::FETCH_ASSOC)) {
				$domain = new Domain(name: $result['name'], id: $result['id'], panelID: $result['panel_id'], a: $result['a'], aaaa: $result['aaaa']);
				$domains[] = $domain;
			}
			return $domains;
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param int $id
	 *
	 * @return bool|\App\Entity\Domain
	 */
	public function findByID(int $id): bool|Domain
	{
		$sql = "
			SELECT id, name, panel_id, a, aaaa
			FROM . " . DatabaseConnection::TABLE_DOMAINS . "
            WHERE id = :id";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: ':id', var: $id);
			$statement->execute();
			if ($result = $statement->fetch(mode: PDO::FETCH_ASSOC)) {
				return new Domain(name: $result['name'], panelID: $result['panel_id'], a: $result['a'], aaaa: $result['aaaa']);
			} else {
				return false;
			}
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param String $name
	 *
	 * @return \App\Entity\Domain|bool
	 */
	public function findByName(string $name): Domain|bool
	{
		$sql = "
			SELECT id, name, panel_id, a, aaaa
			FROM " . DatabaseConnection::TABLE_DOMAINS . "
            WHERE name = :name";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: ':name', var: $name);
			$statement->execute();
			if ($result = $statement->fetch(mode: PDO::FETCH_ASSOC)) {
				return new Domain(name: $result['name'], panelID: $result['panel_id'], a: $result['a'], aaaa: $result['aaaa']);
				
			} else {
				return false;
			}
			
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	
	/**
	 * @param String $name
	 * @param int    $panelID
	 * @param String $a
	 * @param String $aaaa
	 *
	 * @return string|false
	 */
	public function insert(string $name, int $panelID, string $a, string $aaaa): bool|string
	{
		print("here");
		$sql = "
            INSERT INTO " . DatabaseConnection::TABLE_DOMAINS . " (name, panel_id, a, aaaa)
            VALUES (:name, :panel_id, :a, :aaaa)";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: ':name', var: $name);
			$statement->bindParam(param: ':panel_id', var: $panelID);
			$statement->bindParam(param: ':a', var: $a);
			$statement->bindParam(param: ':aaaa', var: $aaaa);
			$statement->execute();
			
			return $this->databaseConnection->getConnection()->lastInsertId();
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param Int    $id
	 * @param String $name
	 * @param int    $panelID
	 * @param String $a
	 * @param String $aaaa
	 *
	 * @return false|int
	 */
	public function update(int $id, string $name, int $panelID, string $a, string $aaaa): bool|int
	{
		$current = $this->findByID(id: $id);
		
		/* doesn't work
		$statement = "
            INSERT INTO domains(id, name, a, aaaa)
            VALUES(:id, :name, :a, :aaaa)
            ON DUPLICATE KEY UPDATE
            	name=COALESCE(VALUES(name), :name),
				a=COALESCE(:a, a),
            	aaaa=COALESCE(:aaaa, aaaa)";
		*/
		
		if (empty($name)) {
			$name = $current['name'];
		}
		if (empty($panelID)) {
			$panelID = $current['panel_id'];
		}
		$panelID = intval(value: $panelID);
		if (empty($a)) {
			$a = $current['a'];
		}
		if (empty($aaaa)) {
			$aaaa = $current['aaaa'];
		}
		
		$sql = "
			UPDATE " . DatabaseConnection::TABLE_DOMAINS . " SET
				name = :name,
				panel_id = :panel_id,
				a = :a,
               	aaaa = :aaaa
            WHERE id = :id";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: 'id', var: $id);
			$statement->bindParam(param: 'name', var: $name);
			$statement->bindParam(param: 'panel_id', var: $panelID);
			$statement->bindParam(param: 'a', var: $a);
			$statement->bindParam(param: 'aaaa', var: $aaaa);
			$statement->execute();
			
			return $statement->rowCount();
		} catch (PDOException $e) {
			print($e->getMessage());
			return false;
		}
	}
	
	
	/**
	 * @param $id
	 *
	 * @return int
	 */
	public function delete($id): int
	{
		$sql = "
            DELETE FROM " . DatabaseConnection::TABLE_DOMAINS . "
            WHERE id = :id";
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->bindParam(param: 'id', var: $id);
			$statement->execute();
			
			return $statement->rowCount();
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
	
	
	/**
	 * @param String $field
	 *
	 * @return int
	 */
	public function getLongestEntry(String $field): int
	{
		$sql = "
            SELECT MAX(LENGTH(" . $field . ")) as length FROM " . DatabaseConnection::TABLE_DOMAINS;
		
		try {
			$statement = $this->databaseConnection->getConnection()->prepare(query: $sql);
			$statement->execute();
			$result =  $statement->fetch();
			return $result['length'];
		} catch (PDOException $e) {
			exit($e->getMessage());
		}
	}
}